@@ -19,6 +19,7 @@ from test.cache.storage.dummy_cache_storage import DummyCacheStorage
 from test.helper_test import uploader_response_mock, SUFFIX, TEST_IMAGE, get_params, TEST_ICON, TEST_DOC, \
     REMOTE_TEST_IMAGE, UTC, populate_large_file, TEST_UNICODE_IMAGE, get_uri, get_method, get_param, \
     cleanup_test_resources_by_tag, cleanup_test_transformation, cleanup_test_resources, EVAL_STR
+from test.test_utils import TEST_ID, TEST_FOLDER
 
 MOCK_RESPONSE = uploader_response_mock()
 
@@ -70,6 +71,10 @@ METADATA_FIELDS = {
     METADATA_FIELD_EXTERNAL_ID_SET: [DATASOURCE_ENTRY_1, DATASOURCE_ENTRY_2]
 }
 
+FD_PID_PREFIX = "fd_public_id_prefix"
+ASSET_FOLDER = "asset_folder"
+DISPLAY_NAME = "test"
+
 disable_warnings()
 
 
@@ -80,24 +85,24 @@ class UploaderTest(unittest.TestCase):
     rbp_params = {
         "use_cache": True,
         "responsive_breakpoints":
-        [
-            {
-                "create_derived": False,
-                "transformation":
+            [
                 {
-                    "angle": 90
+                    "create_derived": False,
+                    "transformation":
+                        {
+                            "angle": 90
+                        },
+                    "format": "gif"
                 },
-                "format": "gif"
-            },
-            {
-                "create_derived": False,
-                "transformation": rbp_trans,
-                "format": rbp_format
-            },
-            {
-                "create_derived": False
-            }
-        ],
+                {
+                    "create_derived": False,
+                    "transformation": rbp_trans,
+                    "format": rbp_format
+                },
+                {
+                    "create_derived": False
+                }
+            ],
         "type": "upload"
     }
 
@@ -157,6 +162,26 @@ class UploaderTest(unittest.TestCase):
         self.assertIn(METADATA_FIELD_UNIQUE_EXTERNAL_ID, result['metadata'])
         self.assertEqual(result['metadata'].get(METADATA_FIELD_UNIQUE_EXTERNAL_ID), METADATA_FIELD_VALUE)
 
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_upload_path_lib_path(self):
+        """Should successfully upload a pathlib.Path file object"""
+        try:
+            import pathlib
+        except ImportError:
+            self.skipTest("pathlib is not supported")
+            return
+
+        path_lib_image_path = pathlib.Path(TEST_IMAGE)
+
+        result = uploader.upload(path_lib_image_path, tags=[UNIQUE_TAG])
+
+        self.assertEqual(result["width"], TEST_IMAGE_WIDTH)
+        self.assertEqual(path_lib_image_path.stem, result["original_filename"])
+
+        result = uploader.upload_large(path_lib_image_path, tags=[UNIQUE_TAG], resource_type="image")
+
+        self.assertEqual(result["width"], TEST_IMAGE_WIDTH)
+
     @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
     def test_upload_unicode_filename(self):
         """Should successfully upload file with unicode characters"""
@@ -218,6 +243,22 @@ class UploaderTest(unittest.TestCase):
         uploader.upload(TEST_IMAGE, tags=[UNIQUE_TAG], **async_option)
         self.assertTrue(get_param(mocker, 'async'))
 
+    @patch('urllib3.request.RequestMethods.request')
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_upload_folder_decoupling(self, mocker):
+        """Should pass folder decoupling params """
+        mocker.return_value = MOCK_RESPONSE
+
+        fd_params = {"public_id_prefix": FD_PID_PREFIX, "asset_folder": ASSET_FOLDER, "display_name": DISPLAY_NAME,
+                     "use_filename_as_display_name": True, "folder": TEST_FOLDER}
+        uploader.upload(TEST_IMAGE, tags=[UNIQUE_TAG], **fd_params)
+
+        self.assertEqual(FD_PID_PREFIX, get_param(mocker, "public_id_prefix"))
+        self.assertEqual(ASSET_FOLDER, get_param(mocker, "asset_folder"))
+        self.assertEqual(DISPLAY_NAME, get_param(mocker, "display_name"))
+        self.assertEqual("1", get_param(mocker, "use_filename_as_display_name"))
+        self.assertEqual(TEST_FOLDER, get_param(mocker, "folder"))
+
     @patch('urllib3.request.RequestMethods.request')
     @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
     def test_ocr(self, mocker):
@@ -387,7 +428,8 @@ P9/AFGGFyjOXZtQAAAAAElFTkSuQmCC\
 
         # Test explicit with metadata
         resource = uploader.upload(TEST_IMAGE, tags=[UNIQUE_TAG])
-        result_metadata = uploader.explicit(resource['public_id'], type="upload", metadata=METADATA_FIELDS, tags=[UNIQUE_TAG])
+        result_metadata = uploader.explicit(resource['public_id'], type="upload", metadata=METADATA_FIELDS,
+                                            tags=[UNIQUE_TAG])
         self.assertIn(METADATA_FIELD_UNIQUE_EXTERNAL_ID, result_metadata['metadata'])
         self.assertEqual(result_metadata['metadata'].get(METADATA_FIELD_UNIQUE_EXTERNAL_ID), METADATA_FIELD_VALUE)
 
@@ -464,6 +506,81 @@ P9/AFGGFyjOXZtQAAAAAElFTkSuQmCC\
         self.assertGreater(result["width"], 1)
         self.assertGreater(result["height"], 1)
 
+    @patch('urllib3.request.RequestMethods.request')
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_create_slideshow_from_manifest_transformation(self, mocker):
+        """Should create slideshow from a manifest transformation"""
+        mocker.return_value = MOCK_RESPONSE
+
+        slideshow_manifest = "w_352;h_240;du_5;fps_30;vars_(slides_((media_s64:aHR0cHM6Ly9y" + \
+                             "ZXMuY2xvdWRpbmFyeS5jb20vZGVtby9pbWFnZS91cGxvYWQvY291cGxl);(media_s64:aH" + \
+                             "R0cHM6Ly9yZXMuY2xvdWRpbmFyeS5jb20vZGVtby9pbWFnZS91cGxvYWQvc2FtcGxl)))"
+
+        uploader.create_slideshow(
+            manifest_transformation={
+                "custom_function": {
+                    "function_type": "render",
+                    "source": slideshow_manifest,
+                }
+            },
+            transformation={"fetch_format": "auto", "quality": "auto"},
+            tags=['tag1', 'tag2', 'tag3']
+        )
+
+        args, _ = mocker.call_args
+
+        self.assertTrue(get_uri(args).endswith('/video/create_slideshow'))
+
+        self.assertEqual("fn_render:" + slideshow_manifest, get_params(args)['manifest_transformation'])
+        self.assertEqual("f_auto,q_auto", get_params(args)['transformation'])
+        self.assertEqual("tag1,tag2,tag3", get_params(args)['tags'])
+
+    @patch('urllib3.request.RequestMethods.request')
+    @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
+    def test_create_slideshow_from_manifest_json(self, mocker):
+        """Should create slideshow from a manifest json"""
+        mocker.return_value = MOCK_RESPONSE
+        slideshow_manifest_json = OrderedDict((
+            ("w", 848),
+            ("h", 480),
+            ("du", 6),
+            ("fps", 30),
+            ("vars", OrderedDict((
+                ("sdur", 500),
+                ("tdur", 500),
+                ("slides", [
+                    {"media": "i:protests9"},
+                    {"media": "i:protests8"},
+                    {"media": "i:protests7"},
+                    {"media": "i:protests6"},
+                    {"media": "i:protests2"},
+                    {"media": "i:protests1"}
+                ])
+            )))
+        ))
+
+        slideshow_manifest_json_str = '{"w":848,"h":480,"du":6,"fps":30,"vars":{"sdur":500,"tdur":500,' + \
+                                      '"slides":[{"media":"i:protests9"},{"media":"i:protests8"},' + \
+                                      '{"media":"i:protests7"},{"media":"i:protests6"},{"media":"i:protests2"},' + \
+                                      '{"media":"i:protests1"}]}}'
+        notification_url = "https://example.com"
+
+        uploader.create_slideshow(
+            manifest_json=slideshow_manifest_json,
+            overwrite=True,
+            public_id=TEST_ID,
+            notification_url=notification_url,
+            upload_preset=API_TEST_PRESET
+        )
+
+        args, _ = mocker.call_args
+
+        self.assertEqual(slideshow_manifest_json_str, get_params(args)["manifest_json"])
+        self.assertEqual("1", get_params(args)["overwrite"])
+        self.assertEqual(TEST_ID, get_params(args)["public_id"])
+        self.assertEqual(notification_url, get_params(args)["notification_url"])
+        self.assertEqual(API_TEST_PRESET, get_params(args)["upload_preset"])
+
     @unittest.skipUnless(cloudinary.config().api_secret, "requires api_key/api_secret")
     def test_tags(self):
         """Should successfully upload file """
@@ -705,8 +822,8 @@ P9/AFGGFyjOXZtQAAAAAElFTkSuQmCC\
 
         # Should accept a dictionary of strings
         acl = OrderedDict((("access_type", "anonymous"),
-                          ("start", "2018-02-22 16:20:57 +0200"),
-                          ("end", "2018-03-22 00:00 +0200")))
+                           ("start", "2018-02-22 16:20:57 +0200"),
+                           ("end", "2018-03-22 00:00 +0200")))
         exp_acl = '[{"access_type":"anonymous","start":"2018-02-22 16:20:57 +0200","end":"2018-03-22 00:00 +0200"}]'
 
         uploader.upload(TEST_IMAGE, access_control=acl)
@@ -717,8 +834,8 @@ P9/AFGGFyjOXZtQAAAAAElFTkSuQmCC\
 
         # Should accept a dictionary of datetime objects
         acl_2 = OrderedDict((("access_type", "anonymous"),
-                            ("start", datetime.strptime("2019-02-22 16:20:57Z", "%Y-%m-%d %H:%M:%SZ")),
-                            ("end", datetime(2019, 3, 22, 0, 0, tzinfo=UTC()))))
+                             ("start", datetime.strptime("2019-02-22 16:20:57Z", "%Y-%m-%d %H:%M:%SZ")),
+                             ("end", datetime(2019, 3, 22, 0, 0, tzinfo=UTC()))))
 
         exp_acl_2 = '[{"access_type":"anonymous","start":"2019-02-22T16:20:57","end":"2019-03-22T00:00:00+00:00"}]'
 
@@ -739,7 +856,7 @@ P9/AFGGFyjOXZtQAAAAAElFTkSuQmCC\
         # Should accept a list of all the above values
         list_of_acl = [acl, acl_2, acl_str]
         # Remove starting "[" and ending "]" in all expected strings and combine them into one string
-        expected_list_of_acl = "[" + ",".join([v[1:-1] for v in(exp_acl, exp_acl_2, exp_acl_str)]) + "]"
+        expected_list_of_acl = "[" + ",".join([v[1:-1] for v in (exp_acl, exp_acl_2, exp_acl_str)]) + "]"
 
         uploader.upload(TEST_IMAGE, access_control=list_of_acl)
         params = get_params(request_mock.call_args[0])
