@@ -794,6 +794,18 @@ class TestUtils(unittest.TestCase):
                                                                                   'level': '3.1'}},
                                    expected_url=VIDEO_UPLOAD_PATH + "vc_h264:basic:3.1/video_id")
 
+        # b_frames=True -> should not add b_frames parameter
+        self.__test_cloudinary_url(public_id="video_id", options={'resource_type': 'video',
+                                                                  'video_codec': {'codec': 'h265', 'profile': 'auto',
+                                                                                  'level': 'auto', 'b_frames': True}},
+                                   expected_url=VIDEO_UPLOAD_PATH + "vc_h265:auto:auto/video_id")
+
+        # should support a b_frames parameter - b_frames=False -> bframes_no
+        self.__test_cloudinary_url(public_id="video_id", options={'resource_type': 'video',
+                                                                  'video_codec': {'codec': 'h265', 'profile': 'auto',
+                                                                                  'level': 'auto', 'b_frames': False}},
+                                   expected_url=VIDEO_UPLOAD_PATH + "vc_h265:auto:auto:bframes_no/video_id")
+
     def test_audio_codec(self):
         # should support a string value
         self.__test_cloudinary_url(public_id="video_id", options={'resource_type': 'video', 'audio_codec': 'acc'},
@@ -934,6 +946,27 @@ class TestUtils(unittest.TestCase):
             result = cloudinary.utils.process_layer(options, "overlay")
             self.assertEqual(expected, result)
 
+    def test_text_layer_style_identifier_variables(self):
+        options = {
+            "transformation": [
+                {
+                    "variables": [
+                        ["$style", "!Arial_12!"],
+                    ]
+                },
+                {
+                    "overlay": {
+                        "text": "hello-world",
+                        "text_style": "$style"
+                    }
+                }
+            ]
+        }
+
+        public_id = "sample"
+        url, _ = cloudinary.utils.cloudinary_url(public_id, **options)
+        self.assertEqual(DEFAULT_UPLOAD_PATH + "$style_!Arial_12!/l_text:$style:hello-world/sample", url)
+
     def test_overlay_error_1(self):
         """ Must supply font_family for text in overlay """
         with self.assertRaises(ValueError):
